#ifndef __UTIL_ARRAY
#define __UTIL_ARRAY

// Also see:
// http://www.optimalworks.net/blog/2007/web-development/javascript/array-detection
Array.Is = function(array) {
  return (array != null) &&
    (typeof(array) == 'object') &&
    ((array.prototype == Array.prototype) ||
     (array.constructor &&
      !array.nodeType &&
      !array.item &&
      array.length != undefined));
};

// Array.ForEach iterates over a collection with a 2-argument function f(name,
// item).  If the collection is an array, ForEach calls f(item, item) for each
// item in the collection:  otherwise, ForEach calls f(index, collection[index])
// for each index in the associative array.
//
// This is very useful for reading data or function arguments where you usually
// want the name and the thing itself to be the same but sometimes want to
// have the name and thing iself be different.
Array.ForEach = function(collection, f, reverse) {
  var is_array = Array.Is(collection);
  for (var name in collection) {
    var value = collection[name];
    if (is_array)
      f(value, value);
    else if (reverse)
      f(value, name);
    else
      f(name, value);
  }
};

// Concatinate two argument lists, only returning a new list if both lists
// are non-empty.
Array.Concat = function(x, y) {
  if (!(x && x.length))
    return y;

  if (!(y && y.length))
    return x;

  return arrayfromargs(x).concat(arrayfromargs(y));
};

#endif  // __UTIL_ARRAY
